/* smartMesh Source Code 
 * Copyright (c) Hasan Ayhan, Tobias Welschenbach
 * 
 * This code will manage the main program for the Arduino as well as the implementation of the
 * menus seen on pfodApp on the Android smartphone.
 * 
 * The program has two main modes (automatic and manual) and two submodes winter and summer that are applicable only
 * when automatic is enabled.
 * The two modes winter and summer work with standard values for the inside and outside temperature and the light.
 * Every time the program goes through the loop function, it checks which modes are activated. Dependent on this and 
 * the calculated temperature and light values from the sensors, it then chooses the action (open or close the window).
 * Per default the program is set to summer automatic mode and behaves like this.
 * The user can switch between summer, winter automatic modes and a manual mode via smartphone pfodApp. Once the user 
 * changes the mode the program detects that change and changes the modes accordingly. In summer and winter automatic
 * modes the user can set his/her own values for inside and outside temperatures which the program then replaces with 
 * the default standard values, giving the user the possiblity to 
 * customize the behaviour of the facade to his own needs. When switched to manual, the user can directly control the 
 * the pumps with the pfodApp and either fill or evacuate the balloons manually.
 * 
 * Besides the loop function there are two important functions fillBalloon() and vaccBalloon() which are called if the 
 * facade has to be either pumped with air
 * or evacuated with the vaccuum pump.
 */

/*
 * include all the libraries needed
 */
#include <SFE_TSL2561.h>
#include <Wire.h>
// Install pfodParser library from http://www.forward.com.au/pfod/pfodParserLibraries/pfodParser.zip
#include <pfodParser.h>
#include <SoftwareSerial.h>     //this will include the software serial port library 

pfodParser parser;  //create Parser instance

 
//=============== global variables for Bluetooth Shield ======================================
int rx_pin = 6;                                  //setting digital pin 6 to be the receiving pin
int tx_pin = 7;                                  //setting the digital pin 7 to be the transmitting pin
SoftwareSerial blueToothSerial(rx_pin, tx_pin);  //creates a new SoftwareSerial object with the
                                                 //selected pins

//==================global variables for pfod Command Management (pfodApp Android) ========================
char cmdPrefix[10];                                              // allocate space for the cmd prefixes
int maxCmdPrefixSize = sizeof(cmdPrefix)/sizeof(cmdPrefix[0])-1; // allow one spot for the terminating null
char* completeCmd;                                               // the cmd send by pfodApp 
char cmd = '\0';                                                 // the current cmd char being processed
byte level = 0;                                                  // the menu level being processed.

//==================global variables for the slider items values========================
double setpointInnen = 0;       //the temperature (inside) value sent from slider (0-45)
int setpointInnenAsInt = 0;     //temperature (inside) of slider in integer (0-9)
double setpointAussen = 0;      //the temperature (outside) value sent from slider (0-45)
int setpointAussenAsInt = 0;    //temperature (outside) of slider in integer (0-9)
                                // max value is 9 so don't need to use a long

//=========================global variables for main program================================================================//
//========================= all pins ===================================================//

//pins for the temperature sensors
int tempPinIndoor = 0;
int tempPinOutdoor = 2;

// pins that control the filling
int pumpPin = 2;                // pump that fills the balloons with air 
int valvePump = 8;              // valve that controls the air pump

// pins that control the valves of the Balloons
int valveBall1 = 3;             // valve balloon 1
int valveBall2 = 4;             // valve balloon 2

// pins that control the emptying of the balloons
int vacPin = 5;                 //vacuum pump
int valveVac  = 9;              //valve of the vacuum pump

// pins with the switches that control when the filling needs to stop. Otherwise the balloons would explode, or destroy the facade
int tasterpin1 = 10;
int tasterpin2 = 11;

//========================all other global variables===============================================//

boolean summer = true;          //global variable for setting summer and winter mode

SFE_TSL2561 light;
boolean gain;                   // Gain setting, 0 = X1, 1 = X16;
unsigned int ms;                // Integration ("shutter") time in milliseconds
int delayTime = 1000;           // 30 sec

//calculated actual values from sensors
float tempIndoor;   
float tempOutdoor;
float luxOutdoor;
int idx;

//global variable for counting vacuum pump time 
int count;                      //always if one of the balloons is pumped increases and sets the duration
                                //of one full pumping to allow to evacuate the balloons with this same duration


//int balloon = 0; // 1 := Balloon 1; 2 := Balloon 2
//boolean standard = true;
boolean automatic = true;       // automatic true, or false if user wants manual function, always per default set to automatic
boolean fillBalloon1 = false;   // true, when it should be filled. false, when it should not be filled
boolean fillBalloon2 = false;
//boolean isFullBalloon1 = false;
//boolean isFullBalloon2 = false;
boolean isEmpty = true;        //true only if both balloons are empty, false only if both are full
int tasterstatus;

// if the user wants to fill or emtpy the balloons on his own, theese booleans need to become true
boolean manualFill = false;
boolean manualVac = false;

//variables that control when the balloons should be opened or closed
// those values are temporary set to compare them with the actual indoor and outdoor temperature. they are equal to the standard temperature values, or to the values the user has set.
double tmpI;
double tmpO;

//our standard values for the summer mode
int standardTempSummerIndoor = 25;
int standardTempSummerOutdoor = 30;

// our standard values for the winter mode
int standardTempWinterIndoor = 25;
int standardTempWinterOutdoor = 19;

// variables used for the brightness of the sunlight
int standardLux = 500; //standard brightness value. The shining of the sunlight on the facade should not be higher than this value to open the window in summer, in winter the opposite


//================================== END OF DECLARATIONS ================================================================================================================//

/*
 * This function is called once on restart of the Arduino to initialize everything (like Bluetooth Shield, light sensor)
 * and to set the pinModes of the pins.
 */
void setup() {
  
  Serial.begin(9600);
  for (int i=5; i>0; i--) {
    Serial.print(i);
    delay(1000);
  }
  Serial.println();
  
  // set pinModes of Bluetooth and init Bluetooth connection
  pinMode(rx_pin, INPUT);                                           //receiving pin as INPUT
  pinMode(tx_pin, OUTPUT);                                          //transmitting pin as OUTPUT
  setBlueToothConnection();                                         //initialize the bluetooth connection
  cmdPrefix[maxCmdPrefixSize] = '\0';                               // make sure we have a null at the end
  
  pinMode(tasterpin1,INPUT);
  pinMode(tasterpin2,INPUT);
  pinMode(pumpPin, OUTPUT);
  pinMode(valveBall1, OUTPUT);
  pinMode(valveBall2, OUTPUT);
  pinMode(vacPin, OUTPUT);
  pinMode(valvePump, OUTPUT);
  pinMode(valveVac, OUTPUT);
  
  //===== init light sensor=========================================//
  light.begin();
  // Get factory ID from sensor:
  // (Just for fun, you don't need to do this to operate the sensor)
  unsigned char ID;
  
  if (light.getID(ID))
  {
    Serial.print("Got factory ID: 0X");
    Serial.print(ID,HEX);
    Serial.println(", should be 0X5X");
  }
  else
  {
    byte error = light.getError();
    printError(error);
  }
  gain = 0;
  unsigned char time = 2;

  Serial.println("Set timing...");
  light.setTiming(gain,time,ms);
  Serial.println("Powerup...");
  light.setPowerUp();
  
 //set the counter____________________________________
 if ((getTasterStatus(tasterpin1) == HIGH) || (getTasterStatus(tasterpin2) == HIGH))  // if balloon is allready full, the counter has to be high, to empty them
 { 
   count = 570;       //this is an apporximate value determined by the times the loop function gets called when pumping the empty balloons completely full
   isEmpty = false;   //if balloons full set isEmpty=false to allow to empty them with vaccBalloon()
 }else{
   count = 0;
 } 
}

/*
 * The most important function which everytime calculates the temperature and light values and checks which mode is activated and decides whether to fill or evacuate the balloons. 
 * It also calls the functions that are responsible for the Menus that are seen on the smartphone screen on pfodApp.
 * It runs infinitely often.
 */
void loop() {
  
  //======================calculate light value in Lux====================================================//
  unsigned int data0, data1;
  
  if (light.getData(data0,data1))
  {

    Serial.print("data0: ");
    Serial.print(data0);
    Serial.print(" data1: ");
    Serial.print(data1);
  
    double lux;    // Resulting lux value
    boolean good;  // True if neither sensor is saturated
    
    // Perform lux calculation:
    good = light.getLux(gain,ms,data0,data1,lux);
    
    // Print out the results:
    Serial.print(" lux: ");
    Serial.print(lux);
    luxOutdoor = lux;
    if (good) Serial.println(" (good)"); else Serial.println(" (BAD)");
  }
  else
  {
    // getData() returned false because of an I2C error, inform the user.
    byte error = light.getError();
    printError(error);
  }
  
  //==========================calculate temperature values by calling getTemperatur function===============//
  Serial.print(" TemperaturIndoor: "); 
  Serial.println(getTemperatur(tempPinIndoor));
  // 1000ms wartem
  //delay(1000);
  Serial.print(" TemperaturOutdoor: "); 
  Serial.println(getTemperatur(tempPinOutdoor));
  // 1000ms wartem
  //delay(1000); //when activated, pfodApp times out while connecting 
  //============================================================================================//
  
  //=====================Code for the pfodApp Menu==============================================//
  byte in = 0;
  if (blueToothSerial.available()) {
    in = blueToothSerial.read(); // read the next char
    byte cmdFirstByte = parser.parse(in); // pass it to the parser
    // parser returns non-zero when a pfod command is fully parsed
    if (cmdFirstByte != 0) { // have parsed a complete msg { to }
      completeCmd = (char*)parser.getCmd(); // set pointer to cmd
      level = 1; // set level
      if ('.' == cmdFirstByte) {
        // pfodApp has connected and sent {.} it is asking for the main menu
        // send back the menu designed
        getMainMenu();
        // now handle commands returned from button clicks 
      } else if('i'==cmdFirstByte) { // user pressed -- Intelligente Steuerung 
        // ============= SUBMENU MODE =====================
        getSubMenuMode();
        //blueToothSerial.print(F("{}")); // edit this to update existing menu or send back sub-menu
        // always send back a pfod msg otherwise pfodApp will disconnect.
      } else if('m'==cmdFirstByte) { // user pressed -- Manuell 
        // ============= SUBMENU Manuell =====================
        getSubMenuManual(); 
        //blueToothSerial.print(F("{}")); // edit this to update existing menu or send back sub-menu
        // always send back a pfod msg otherwise pfodApp will disconnect.
      } else if('d'==cmdFirstByte) { // user pressed -- Sensoren
        // ============= SUBMENU Sensoren =====================
       //getSubMenuSensoren();
        blueToothSerial.print(F("{}")); // edit this to update existing menu or send back sub-menu
        // always send back a pfod msg otherwise pfodApp will disconnect.
      } else {
        // unknown command
        blueToothSerial.print(F("{}")); // always send back a pfod msg otherwise pfodApp will disconnect.
      }
    }
    cmdFirstByte = 0; //have processed this cmd now so clear it and wait for next one
  }
  
  //========================================================================================================================//
  //=============================== Manage the main program (deciding modes and if window opens or closes ==================//
  
  //if user did not press manual or pressed again automatic mode
  if (automatic)
  {
       //if user has set summer mode in pfodApp or if he did not set anything yet (per default summer mode on)
       if(summer){
        if(setpointInnen != 0) {                        //if user has set an inside temp value, save this in tmpI to compare to the actual temperature later
          tmpI = setpointInnen;
        }else{
          tmpI = standardTempSummerIndoor; //25         //if user has not set an inside temp value yet, then set tmpI to the fixed standard temp value
        }
        if(setpointAussen != 0) {                       //if user has set an outside temp value, save this in tmpO to compare to the actual temperature later
          tmpO = setpointAussen;
        }else{
          tmpO = standardTempSummerOutdoor; //30        //if user has not set an outside temp value yet, then set tmpO to the fixed standard temp value
        }     
        Serial.println(tmpI);
        Serial.println(tmpO);
        
        //calculate actual temperature values with sensors
        tempIndoor = getTemperatur(tempPinIndoor);
        tempOutdoor = getTemperatur(tempPinOutdoor);
        
        //only close when it is too hot outside and not too warm inside to save cold air inside 
        if ((tempIndoor <= tmpI) && (tempOutdoor >= tmpO)){
            fillBalloon(); // rufe nun die funktion zum befüllen der balloons auf
        }
        
        //open when it is too hot inside and not too warm outside and if it is not too bright to get fresh air  
        if ((tempIndoor > tmpI) && (luxOutdoor <= standardLux) && (tempOutdoor < tmpO)) {
           vaccBalloon();    
        }
       }
   
        //if user has set winter mode in pfodApp
        if(!summer){
         if(setpointInnen != 0) {
           tmpI = setpointInnen;
         }else{
           tmpI = standardTempWinterIndoor; //25
         }
         if(setpointAussen != 0) {
           tmpO = setpointAussen;
         }else{
           tmpO = standardTempWinterOutdoor; //19
         }
        
        tempIndoor = getTemperatur(tempPinIndoor);
        tempOutdoor = getTemperatur(tempPinOutdoor);
    
        // if the outdoor temperature is too cold then close the window independent of the actual inside temperature
        if ((tempOutdoor <= tmpO)){
            fillBalloon(); // rufe nun die funktion zum befüllen der balloons auf
        }
        
        //if it is too warm inside and not too cold outside and also enough light outside then 
        if ((tempIndoor > tmpI) && (luxOutdoor >= standardLux) && (tempOutdoor > tmpO)){
            //isEmpty = false;
            vaccBalloon();
        }
       }
  }
  
  // the user has set the automatic to false, and he wants to fill, or empty the balloon manually
  if (!automatic){
    if(manualFill){
      fillBalloon();
    }
    if(manualVac){
      vaccBalloon();
    }
  }

}
//end of loop
//---------------------------------------------------------------------------------------------

//==============================================================================================//

/*
 * Function to fill the balloons. If one, or both balloons should get filled then the pump turns on, and the valves 
 * are set to open. But if both balloons are already full, the pump is not be powered anymore and the valves are closed.
 */
void fillBalloon()
{
  //count = 570;       //manually set the counter high again so that later the balloons can be evacuated again with vaccBalloon()
    
  //if the balloon is not allready full it's getting filled  
  if (getTasterStatus(tasterpin1) == LOW) // wenn der taster nicht gedrückt, also der balloon nicht ganz, oder gar nicht voll ist, setze fillBalloon auf wahr
         {
           Serial.println("Balloon 1 true");
           fillBalloon1 = true;
           //if vac pump is on, then first deactivate valves of vac pump and the vac Pin
           digitalWrite(vacPin, LOW);
           digitalWrite(valveVac, LOW);
           digitalWrite(pumpPin, HIGH);           //power on the pump
           digitalWrite(valveBall1, HIGH);        //open the balloons valve
           digitalWrite(valvePump, HIGH);         //open the pumps valve
         }else{
           digitalWrite(valveBall1, LOW);         //close the balloon's valve
           //isFullBalloon1 = true;
           fillBalloon1 = false; // but if the balloon is already full, this boolean is set to false
       }
   
   //if the balloon is not allready full it's getting filled      
   if (getTasterStatus(tasterpin2) == LOW) // same settings like above  
         {
           Serial.println("Balloon 2 true");
           fillBalloon2 = true;
           //if vac pump is on, then first deactivate valves of vac pump and the vac Pin
           digitalWrite(vacPin, LOW);
           digitalWrite(valveVac, LOW);
           digitalWrite(pumpPin,HIGH);
           digitalWrite(valveBall2, HIGH);
           digitalWrite(valvePump, HIGH);
         }else{
           digitalWrite(valveBall2, LOW);
           //isFullBalloon2 = true;
           fillBalloon2 = false; // siehe oben
         }
  
    //everytime we are on this place in the code count increase to have a value for the amount of times the vaccBalloon function should be called to empty the balloons later
    if( (fillBalloon1) || (fillBalloon2)){
       count++;
    }
  
    // If both balloons are full, the pump is turned off and it's valve is getting to be closed 
    if ((!fillBalloon1) && (!fillBalloon2))
    {  
       Serial.print("Counter nach Aufpumpen: ");
       Serial.print(count);
       digitalWrite(pumpPin, LOW);
       digitalWrite(valvePump, LOW);
       isEmpty = false; //both balloons are full. This boolean is needed in the vacBalloon() function. Because, the ballooons should only get emptied when they are full
       
       //if user has not switched to automatic mode again after manual mode turn manualFill= false to ensure that fillBalloon will not be called anymore 
       if((!automatic) && (manualFill == true)){
         manualFill = false;
       }
     }
}

/* 
 * This function empties the balloons iff isEmpty is set false and the count is not zero.
 * This ensures that the balloons are only emptied if they were full before, since isEmpty is only
 * set false after finishing with pumping in fillBalloon(). (and also in setup when restarted, but only if
 * the balloons are already full because the tasters are activated.
 * 
 */

void vaccBalloon()  

{
  Serial.println("abpumpen");
  Serial.println("isEmpty: ");
  Serial.println(isEmpty);
  
  if ((!isEmpty) && (count > 0) )
  {  
    Serial.println("pumpt ab");
    digitalWrite(valveBall1, HIGH);// zuerst werden alle ventile außer das der pumpe geöffnet und die vakuum pumpe wird an gemacht
    digitalWrite(valveBall2, HIGH);
    digitalWrite(valveVac, HIGH);
    digitalWrite(valvePump, LOW);
    digitalWrite(vacPin, HIGH);
  
    //decrease the counter 
    count--; 
    Serial.print("count: ");
    Serial.print(count);
 
   }else{
     isEmpty = true; //after evacuating isEmpty is set true till it becomes false again after pumping
     digitalWrite(valveBall1, LOW); // nach der delayTime werden alle ventile wieder verschlossen und alle pumpe deaktiviert
     digitalWrite(valveBall2, LOW);
     digitalWrite(valveVac, LOW);
     digitalWrite(valvePump, LOW);
     digitalWrite(vacPin, LOW);
     manualVac = false;          
     Serial.print("pumpt nicht ab");
   }
   
}

// function to get taster status ( if pressed or not)
int getTasterStatus(int tasterPin)
{
  //Serial.print("TasterPin");
  Serial.println(tasterPin);
  tasterstatus = digitalRead(tasterPin);
  return(tasterstatus);
}

// error function for light sensor I2C interface 
void printError(byte error)
  // If there's an I2C error, this function will
  // print out an explanation.
{
  Serial.print("I2C error: ");
  Serial.print(error,DEC);
  Serial.print(", ");
  
  switch(error)
  {
    case 0:
      Serial.println("success");
      break;
    case 1:
      Serial.println("data too long for transmit buffer");
      break;
    case 2:
      Serial.println("received NACK on address (disconnected?)");
      break;
    case 3:
      Serial.println("received NACK on data");
      break;
    case 4:
      Serial.println("other error");
      break;
    default:
      Serial.println("unknown error");
  }
}

// Temperature function
float getTemperatur(int tempPin)
{
  float temperatur;
 
  temperatur = 0;
  // for better result do 5 mesurings and sum them
  for(idx = 0; idx<5; idx++)
  {
    temperatur = temperatur + analogRead(tempPin);
    // delaytime between mesuring 20ms
    delay(20);
  }
  //to get the temperatur divide by 5 and 2
  temperatur = temperatur / 5 / 2; // <-- the 2 can be replaced with a more exact value
  return(temperatur);
}
//====================end of main program code================================================//




//====================pfodApp command handlers and Submenu code===============================//
//==================== See pfod specification online to better understand the protocol =======//
/**
 * display Main Menu screen on pfodApp
 */
void getMainMenu() {
// start a Menu screen pfod message
        blueToothSerial.print(F("{."));
// send menu prompt
        blueToothSerial.print(F("<+4>SmartMesh\nControl "));
// send menu buttons
        blueToothSerial.print(F("|i~<bg s><bl><+4>Intelligente Steuerung "));
        blueToothSerial.print(F("|m~<bg w><bl><+4>Manuell "));
// close pfod message
        blueToothSerial.print(F("}"));
 // ============ end of sending pfod menu message ===========
}

/**
 * Handle the commands starting with 'i'
 */
void getSubMenuMode(){
  // strip off the 'i' and set cmd to next char
  // save 'i' in cmdPrefix for building sub-menu cmds
  if (!updateCmdCharToNextLevel()) {
    return; // error
  }
  if (cmd == '\0') { // cmd was just 'i' so return Submenu Mode

    // start a Menu screen pfod message
    blueToothSerial.print(F("{."));
    // send menu prompt
    // send menu buttons
    blueToothSerial.print(F("|"));
    blueToothSerial.print((char*)cmdPrefix); // add prefix to cmd to get is
    blueToothSerial.print(F("s~<bg s><n><+5>SOMMER"));
    blueToothSerial.print(F("|"));
    blueToothSerial.print((char*)cmdPrefix); // add prefix to cmd to get iw
    blueToothSerial.print(F("w~<bg s><n><+5>WINTER"));
    //send slider items for Innentemperatur and Außentemperatur
    blueToothSerial.print(F("|"));
    blueToothSerial.print((char*)cmdPrefix);
    blueToothSerial.print(F("t~Temperatur Innen `0`9~ °C~5"));
    blueToothSerial.print(F("|"));
    blueToothSerial.print((char*)cmdPrefix);
    blueToothSerial.print(F("T~Temperatur Außen `0`9~ °C~5"));
    // close pfod message
    blueToothSerial.print(F("}")); 
   } else if(cmd == 's') {
      // next cmd char was not null so process that cmd
      //set mode variable to summer to activate summer mode as automatic mode
      automatic = true;
      summer = true;
      returnEmptyResponse();  //just to send 
      //getSubMenuModeSummer();
   } else if(cmd == 'w') {
    // set mode variable to winter to activate winter mode as automatic mode
      automatic = true; 
      summer = false;
      returnEmptyResponse();
     // getSubMenuModeWinter();
   } else if (cmd == 't'){
       automatic = true;
      
      // pick up new value from slider Temperatur Innen
      long value1 = 0;
      byte* argPtr = parser.getFirstArg();
      argPtr = parser.parseLong(argPtr, &value1); 
      // expects a long pointer, returns ptr to next arg
      // now value has slider value in it
      // set it where you want to
      setpointInnenAsInt = value1; // truncates from long to int
      // now update the Setpoint
      setpointInnen = 5 * setpointInnenAsInt; // update Setpoint using scale resolving value (0-45)
      // you can write the value to a debug output via USB
      Serial.println(setpointInnen);
      blueToothSerial.print(F("{}")); // otherwise pfodApp times out and disconnects
   } else if (cmd == 'T'){
     
     automatic = true;     
      // pick up new value from slider Temperatur Außen
      long value2 = 0;
      byte* argPtr = parser.getFirstArg();
      argPtr = parser.parseLong(argPtr, &value2); 
      // expects a long pointer, returns ptr to next arg
      // now value has slider value in it
      // set it where you want to
      setpointAussenAsInt = value2; // truncates from long to int
      // now update the Setpoint
      setpointAussen = 5 * setpointAussenAsInt; // update Setpoint using scale resolving value (0-45)
      // you can write the value to a debug output via USB
      Serial.println(setpointAussen);
      returnEmptyResponse(); // otherwise pfodApp times out and disconnects
      
    } else {
     // don't recognise this command just return {}
      returnEmptyResponse();
   }
}

/**
 * Handle the commands starting with 'm'
 */
void getSubMenuManual(){
  if (!updateCmdCharToNextLevel()) {
    return; // error
  }
  if (cmd == '\0') {
        blueToothSerial.print(F("{."));
       // send menu prompt
        blueToothSerial.print(F("<bg s><bk><+5>Manuelle Steuerung der Pumpen "));
       // send menu buttons
         
        blueToothSerial.print(F("|"));
        blueToothSerial.print((char*)cmdPrefix); // add prefix to cmd m to get mL
        blueToothSerial.print(F("L~<bg g><+5><b>LUFT ON"));
        blueToothSerial.print(F("|"));
        blueToothSerial.print((char*)cmdPrefix); // add prefix to cmd m to get ml
        blueToothSerial.print(F("l~<bg r><+5><b>LUFT OFF"));
        blueToothSerial.print(F("|"));
        blueToothSerial.print((char*)cmdPrefix); // add prefix to cmd m to get mV
        blueToothSerial.print(F("V~<bg s><n><+5><b>Vakuum ON"));
        blueToothSerial.print(F("|"));
        blueToothSerial.print((char*)cmdPrefix); // add prefix to cmd to get mv
        blueToothSerial.print(F("v~<bg s><n><+5><b>Vakuum OFF"));
        // close pfod message
        blueToothSerial.print(F("}"));
  } else {
    if(cmd == 'L') {
    // next cmd char was not null so process that cmd
    // turn air pump on and fill the balloons till they are full 
      automatic = false;
      manualFill = true;
      returnEmptyResponse();
    //fillBalloon();
    //getSubMenuModeSummer();
  } else if(cmd == 'l'){
      //turn air pump off
      //not in this build of the software yet because we haven't defined half empty states of the balloon, only full or empty
      returnEmptyResponse();  //so that it does not disconnect of timeout
  } else if(cmd == 'V'){
      //turn vacuum pump on
      automatic = false;
      manualVac = true;
    // vaccBalloon();
      returnEmptyResponse();  
  } else if(cmd == 'v'){
      //turn vacuum pump off
      //not in this build of the software yet because we haven't defined half empty states of the balloon, only full or empty
      returnEmptyResponse();
  } else {
    // unknown command
    returnEmptyResponse(); // always send back a pfod msg otherwise pfodApp will disconnect.
    } 
 }
   
}
//returns empty Response so that the Arduino (pfodDevice) does not disconnect
void returnEmptyResponse() {
  blueToothSerial.print(F("{}"));
}

/**
 * update the global cmd var with the next char in the complete command
 * also updates the cmdPrefix used for build cmd values to return to pfodApp
 * also increment and check the global level var
 */
bool updateCmdCharToNextLevel() {
  if (level > maxCmdPrefixSize) {
    blueToothSerial.print(F("{.Menu Nesting exceeds code storage|.~Return to main Menu}"));
    return false;
  } // else ok to copy  
  cmd = *(completeCmd+level); // cmd may be null
  strncpy((char*)cmdPrefix, completeCmd, level); // cmdPrefix is global
  *(cmdPrefix+level) = '\0'; // terminate str always
  level++;
  return true;
}



//=======================Bluetooth Init=============================================================

void setBlueToothConnection()
{
    blueToothSerial.begin(38400);                           // Set BluetoothBee BaudRate to default baud rate 38400
    blueToothSerial.print("\r\n+STWMOD=0\r\n");             // set the bluetooth work in slave mode
    blueToothSerial.print("\r\n+STNA=SmartMesh\r\n");       // set the bluetooth name as "SmartMesh"
    blueToothSerial.print("\r\n+STOAUT=1\r\n");             // Permit Paired device to connect me
    blueToothSerial.print("\r\n+STAUTO=0\r\n");             // Auto-connection should be forbidden here
    delay(2000);                                            // This delay is required.
    blueToothSerial.print("\r\n+INQ=1\r\n");                // make the slave bluetooth inquirable
    Serial.println("The slave bluetooth is inquirable!");
    delay(2000);                                            // This delay is required.
    blueToothSerial.flush();
    blueToothSerial.print("Bluetooth connection established correctly!"); // if connection is successful then print to the master device
  
}


